# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AuthIntegrationType(str, Enum):
    """
    AuthIntegrationType
    """

    """
    allowed enum values
    """
    GOOGLE_ACCOUNTS = 'google_accounts'
    WINDCHILL = 'windchill'
    TEAMWORK_CLOUD = 'teamwork_cloud'
    DASSAULT_3D_PASSPORT = 'dassault_3d_passport'
    PING_ID = 'ping_id'
    MICROSOFT_ENTRA = 'microsoft_entra'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AuthIntegrationType from a JSON string"""
        return cls(json.loads(json_str))
