# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AccessRelation(str, Enum):
    """
    PUBLIC: Relation types that can be assigned between a subject and a resource by a user of Istari.
    """

    """
    allowed enum values
    """
    VIEWER = 'viewer'
    EDITOR = 'editor'
    OWNER = 'owner'
    ADMINISTRATOR = 'administrator'
    EXECUTOR = 'executor'
    UPSTREAMREMOTEOWNER = 'upstreamremoteowner'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AccessRelation from a JSON string"""
        return cls(json.loads(json_str))
